from numpy import *
from numpy.linalg import *
from numpy.fft import *
from scipy.special import gamma, genlaguerre
from matplotlib.pyplot import *


V0 = 8.0
beta = 0.25
eps = 1.0
nu = sqrt(8.0*V0 / (beta**2 * eps**4))
nstates = int(floor((nu - 1)/2.0))

M = lambda x: V0*(exp(-2.0*beta*x) - 2.0*exp(-beta*x))

def Phi_n(n, x):
    Nn = sqrt( beta * (nu-2.0*n-1.0)*gamma(n+1) / gamma(nu-n) )
    z = nu*exp(-beta*x)
    s = 0.5*(nu - 2.0*n - 1.0)
    Lp = genlaguerre(n, 2*s)
    L = polyval(Lp, z)
    return Nn * z**s * exp(-0.5*z) * L

def E_n(n):
    return -0.5*(sqrt(2.0*V0) - eps**2*beta*(n+0.5))**2


En = array([E_n(n) for n in xrange(nstates+1)])


N = 2048

###############################################
#                                             #
# TODO: Stellen Sie die Fourier Matrix F auf, #
#       F soll unitaer skaliert sein          #
#                                             #
###############################################

# Check that F is unitary
# abs(eye(N) - dot(conj(F).T,F)).max()


# Compute x grid (position space)
x = ones(N)

####################################
#                                  #
# TODO: Berechnen Sie das x Gitter #
#                                  #
####################################


# Compute k grid (Fourier space)
k = ones(N)

####################################
#                                  #
# TODO: Berechnen Sie das k Gitter #
#                                  #
####################################


H = eye(N)

######################################
#                                    #
# TODO: Stellen Sie die Matrix H auf #
#                                    #
######################################


# Compute Eigenvalues and Eigenvectors
ew = zeros(N)
ev = eye(N)

####################################################
#                                                  #
# TODO: Berechnen Sie Eigenwerte und Eigenvektoren #
#                                                  #
####################################################

n = arange(nstates+1)

figure()
plot(n, real(ew)[:nstates+1], "-bo")
plot(En, "-ro")
grid(True)
xlabel(r"$n$")
ylabel(r"$E_n$")
savefig("morse_eigenvalues.png")


figure()
semilogy(n, abs(En - real(ew)[:nstates+1]), "-bo")
grid(True)
xlabel(r"$n$")
ylabel(r"Error $|E_{\textrm{exact}} - E_{\textrm{numerical}}|$")
savefig("morse_eigenvalues_error.png")


# Compute numerical Eigenfunctions
Psi = []
for n in xrange(nstates+1):
    psi = ones_like(x)

    ####################################################
    #                                                  #
    # TODO: Berechnen Sie die Eigenfunktionen psi_n(x) #
    #                                                  #
    ####################################################

    Psi.append(psi)

# Compute exact Eigenfunctions
Phi = [Phi_n(n, x) for n in xrange(nstates+1)]


# Plot Eigenfunctions
figure(figsize=(12,8))
colors = 10*"bgrcmy"
fill_between(x, M(x), -10*V0, alpha=0.3, color="k")
plot(x, M(x), "k-")

# Numerical Solution
for n, (en, psi) in enumerate(zip(ew, Psi)):
    plot(x, en + conj(psi)*psi, label=r"$\psi_{%d}$" % n, color=colors[n])

# Exact Solution
for n, (en, phi) in enumerate(zip(En, Phi)):
    plot(x, en + conj(phi)*phi, '--k')

grid(True)
xlim((-2, 12))
ylim((-V0-1, 2))
legend(loc='lower right', ncol=3, framealpha=0.8)
xlabel(r"$x$")
ylabel(r"$\psi_n(x)$")
savefig("morse_eigenfunctions.png")


# Plot exakt Eigenfunctions only
figure(figsize=(12,8))
colors = 10*"bgrcmy"

fill_between(x, M(x), -10*V0, alpha=0.3, color="k")
plot(x, M(x), "k-")

# Exact Solution
for n, (en, phi) in enumerate(zip(En, Phi)):
    fill_between(x, en + conj(phi)*phi, en, alpha=0.4, color=colors[n])
    plot(x, en + conj(phi)*phi, label=r"$\phi_{%d}$" % n, color=colors[n])

grid(True)
xlim((-2, 12))
ylim((-V0-1, 2))
legend(loc='lower right', ncol=3, framealpha=0.8)
xlabel(r"$x$")
ylabel(r"$\phi_n(x)$")
savefig("morse_eigenfunctions_exact.png")


# Compute Error in the approximation of Eigenfunctions
error = []

###########################################################
#                                                         #
# TODO: Berechnen Sie den punktweisen Fehler fuer jedes n #
#       und speichern Sie die Werte in der Liste 'error'  #
#                                                         #
###########################################################


# Plot the Error
figure(figsize=(12,8))

for n, err in enumerate(error):
    semilogy(x, err, label=r"$\psi_{%d}$" % n)

grid(True)
xlim((-2,8))
legend(loc='upper right', ncol=3)
xlabel(r"$x$")
ylabel(r"Error $||\phi_n(x)| - |\psi_n(x)||$")
savefig("morse_eigenfunctions_error.png")
